@@ -1,13 +1,13 @@
 /* Test file for mpfr_cmp2.
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999, 2000, 2001, 2002, 2003, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
 The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MPFR Library is distributed in the hope that it will be useful, but
@@ -16,9 +16,9 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -51,37 +51,37 @@ worst_cases (void)
 {
   mpfr_t x, y;
   unsigned int i, j, k, b, expected;
-  mp_prec_t l;
+  mpfr_prec_t l;
 
   mpfr_init2 (x, 200);
   mpfr_init2 (y, 200);
 
-  mpfr_set_ui (y, 1, GMP_RNDN);
+  mpfr_set_ui (y, 1, MPFR_RNDN);
   for (i = 1; i < MPFR_PREC(x); i++)
     {
-      mpfr_set_ui (x, 1, GMP_RNDN);
-      mpfr_div_2exp (y, y, 1, GMP_RNDN); /* y = 1/2^i */
+      mpfr_set_ui (x, 1, MPFR_RNDN);
+      mpfr_div_2exp (y, y, 1, MPFR_RNDN); /* y = 1/2^i */
 
       l = 0;
       if (mpfr_cmp2 (x, y, &l) <= 0 || l != 1)
         {
           printf ("Error in mpfr_cmp2:\nx=");
-          mpfr_out_str (stdout, 2, 0, x, GMP_RNDN);
+          mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
           printf ("\ny=");
-          mpfr_out_str (stdout, 2, 0, y, GMP_RNDN);
-          printf ("\ngot %lu instead of 1\n", l);
+          mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+          printf ("\ngot %lu instead of 1\n", (unsigned long) l);
           exit (1);
         }
 
-      mpfr_add (x, x, y, GMP_RNDN); /* x = 1 + 1/2^i */
+      mpfr_add (x, x, y, MPFR_RNDN); /* x = 1 + 1/2^i */
       l = 0;
       if (mpfr_cmp2 (x, y, &l) <= 0 || l != 0)
         {
           printf ("Error in mpfr_cmp2:\nx=");
-          mpfr_out_str (stdout, 2, 0, x, GMP_RNDN);
+          mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
           printf ("\ny=");
-          mpfr_out_str (stdout, 2, 0, y, GMP_RNDN);
-          printf ("\ngot %lu instead of 0\n", l);
+          mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+          printf ("\ngot %lu instead of 0\n", (unsigned long) l);
           exit (1);
         }
     }
@@ -89,7 +89,7 @@ worst_cases (void)
   for (i = 0; i < 64; i++) /* |u| = i */
     {
       mpfr_urandomb (x, RANDS);
-      mpfr_set (y, x, GMP_RNDN);
+      mpfr_set (y, x, MPFR_RNDN);
       set_bit (x, i + 1, 1);
       set_bit (y, i + 1, 0);
       for (j = 0; j < 64; j++) /* |v| = j */
@@ -110,10 +110,11 @@ worst_cases (void)
               if (l != expected)
                 {
                   printf ("Error in mpfr_cmp2:\nx=");
-                  mpfr_out_str (stdout, 2, 0, x, GMP_RNDN);
+                  mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
                   printf ("\ny=");
-                  mpfr_out_str (stdout, 2, 0, y, GMP_RNDN);
-                  printf ("\ngot %lu instead of %u\n", l, expected);
+                  mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+                  printf ("\ngot %lu instead of %u\n",
+                          (unsigned long) l, expected);
                   exit (1);
                 }
               set_bit (x, i + j + k + 2, 0);
@@ -124,10 +125,11 @@ worst_cases (void)
               if (l != expected)
                 {
                   printf ("Error in mpfr_cmp2:\nx=");
-                  mpfr_out_str (stdout, 2, 0, x, GMP_RNDN);
+                  mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
                   printf ("\ny=");
-                  mpfr_out_str (stdout, 2, 0, y, GMP_RNDN);
-                  printf ("\ngot %lu instead of %u\n", l, expected);
+                  mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+                  printf ("\ngot %lu instead of %u\n",
+                          (unsigned long) l, expected);
                   exit (1);
                 }
             }
@@ -142,7 +144,7 @@ static void
 tcmp2 (double x, double y, int i)
 {
   mpfr_t xx, yy;
-  mp_prec_t j;
+  mpfr_prec_t j;
 
   if (i == -1)
     {
@@ -152,17 +154,17 @@ tcmp2 (double x, double y, int i)
         i = (int) (__gmpfr_floor_log2 (x) - __gmpfr_floor_log2 (x - y));
     }
   mpfr_init2(xx, 53); mpfr_init2(yy, 53);
-  mpfr_set_d (xx, x, GMP_RNDN);
-  mpfr_set_d (yy, y, GMP_RNDN);
+  mpfr_set_d (xx, x, MPFR_RNDN);
+  mpfr_set_d (yy, y, MPFR_RNDN);
   j = 0;
   if (mpfr_cmp2 (xx, yy, &j) == 0)
     {
       if (x != y)
         {
           printf ("Error in mpfr_cmp2 for\nx=");
-          mpfr_out_str (stdout, 2, 0, xx, GMP_RNDN);
+          mpfr_out_str (stdout, 2, 0, xx, MPFR_RNDN);
           printf ("\ny=");
-          mpfr_out_str (stdout, 2, 0, yy, GMP_RNDN);
+          mpfr_out_str (stdout, 2, 0, yy, MPFR_RNDN);
           printf ("\ngot sign 0 for x != y\n");
           exit (1);
         }
@@ -170,10 +172,10 @@ tcmp2 (double x, double y, int i)
   else if (j != (unsigned) i)
     {
       printf ("Error in mpfr_cmp2 for\nx=");
-      mpfr_out_str (stdout, 2, 0, xx, GMP_RNDN);
+      mpfr_out_str (stdout, 2, 0, xx, MPFR_RNDN);
       printf ("\ny=");
-      mpfr_out_str (stdout, 2, 0, yy, GMP_RNDN);
-      printf ("\ngot %lu instead of %d\n", j, i);
+      mpfr_out_str (stdout, 2, 0, yy, MPFR_RNDN);
+      printf ("\ngot %lu instead of %d\n", (unsigned long) j, i);
       exit (1);
     }
   mpfr_clear(xx); mpfr_clear(yy);
@@ -183,7 +185,7 @@ static void
 special (void)
 {
   mpfr_t x, y;
-  mp_prec_t j;
+  mpfr_prec_t j;
 
   mpfr_init (x); mpfr_init (y);
 
@@ -202,7 +204,7 @@ special (void)
       printf ("y=");
       mpfr_print_binary (y);
       puts ("");
-      printf ("got %lu, expected 1\n", j);
+      printf ("got %lu, expected 1\n", (unsigned long) j);
       exit (1);
     }
 
@@ -215,7 +217,7 @@ special (void)
       printf ("Error in mpfr_cmp2:\n");
       printf ("x="); mpfr_print_binary(x); puts ("");
       printf ("y="); mpfr_print_binary(y); puts ("");
-      printf ("got %lu, expected 32\n", j);
+      printf ("got %lu, expected 32\n", (unsigned long) j);
       exit (1);
     }
 
@@ -228,7 +230,7 @@ special (void)
       printf ("Error in mpfr_cmp2:\n");
       printf ("x="); mpfr_print_binary(x); puts ("");
       printf ("y="); mpfr_print_binary(y); puts ("");
-      printf ("got %lu, expected 164\n", j);
+      printf ("got %lu, expected 164\n", (unsigned long) j);
       exit (1);
     }
 
@@ -242,13 +244,13 @@ special (void)
       printf ("Error in mpfr_cmp2:\n");
       printf ("x="); mpfr_print_binary(x); puts ("");
       printf ("y="); mpfr_print_binary(y); puts ("");
-      printf ("got %lu, expected 127\n", j);
+      printf ("got %lu, expected 127\n", (unsigned long) j);
       exit (1);
     }
 
   /* bug found by Nathalie Revol, 2 Apr 2001 */
   mpfr_set_prec (x, 65); mpfr_set_prec (y, 65);
-  mpfr_set_ui (x, 5, GMP_RNDN);
+  mpfr_set_ui (x, 5, MPFR_RNDN);
   mpfr_set_str_binary (y, "0.10011111111111111111111111111111111111111111111111111111111111101E3");
   j = 0;
   if (mpfr_cmp2(x, y, &j) <= 0 || j != 63)
@@ -256,7 +258,7 @@ special (void)
       printf ("Error in mpfr_cmp2:\n");
       printf ("x="); mpfr_print_binary(x); puts ("");
       printf ("y="); mpfr_print_binary(y); puts ("");
-      printf ("got %lu, expected 63\n", j);
+      printf ("got %lu, expected 63\n", (unsigned long) j);
       exit (1);
     }
 
@@ -270,7 +272,7 @@ special (void)
       printf ("Error in mpfr_cmp2:\n");
       printf ("x="); mpfr_print_binary(x); puts ("");
       printf ("y="); mpfr_print_binary(y); puts ("");
-      printf ("got %lu, expected 63\n", j);
+      printf ("got %lu, expected 63\n", (unsigned long) j);
       exit (1);
     }
 
@@ -284,13 +286,13 @@ special (void)
       exit (1);
     }
 
-  mpfr_set_prec (x, 2 * BITS_PER_MP_LIMB);
-  mpfr_set_prec (y, BITS_PER_MP_LIMB);
-  mpfr_set_ui (x, 1, GMP_RNDN); /* x = 1 */
-  mpfr_set_ui (y, 1, GMP_RNDN);
-  mpfr_nextbelow (y);            /* y = 1 - 2^(-BITS_PER_MP_LIMB) */
+  mpfr_set_prec (x, 2 * GMP_NUMB_BITS);
+  mpfr_set_prec (y, GMP_NUMB_BITS);
+  mpfr_set_ui (x, 1, MPFR_RNDN); /* x = 1 */
+  mpfr_set_ui (y, 1, MPFR_RNDN);
+  mpfr_nextbelow (y);            /* y = 1 - 2^(-GMP_NUMB_BITS) */
   mpfr_cmp2 (x, y, &j);
-  if (mpfr_cmp2 (x, y, &j) <= 0 || j != BITS_PER_MP_LIMB)
+  if (mpfr_cmp2 (x, y, &j) <= 0 || j != GMP_NUMB_BITS)
     {
       printf ("Error in mpfr_cmp2 (2)\n");
       exit (1);
