@@ -1,24 +1,24 @@
 /* Test file for mpfr_cmp_ui and mpfr_cmp_si.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #ifdef TCMP_UI_CHECK_NAN
 
@@ -88,6 +88,126 @@ check_nan (void)
   mpfr_clear (x);
 }
 
+/* Since mpfr_cmp_ui and mpfr_cmp_si are also implemented by a macro
+   with __builtin_constant_p for GCC, check that side effects are
+   handled correctly. */
+static void
+check_macros (void)
+{
+  mpfr_t x;
+  int c;
+
+  mpfr_init2 (x, 32);
+
+  c = 0;
+  mpfr_set_ui (x, 17, MPFR_RNDN);
+  if (mpfr_cmp_ui (x, 17) != 0)
+    {
+      printf ("Error 1 on mpfr_cmp_ui(x,17) in check_macros\n");
+      exit (1);
+    }
+  if (mpfr_cmp_ui (x, (c++, 17)) != 0)
+    {
+      printf ("Error 2 on mpfr_cmp_ui(x,17) in check_macros\n");
+      exit (1);
+    }
+  if (c != 1)
+    {
+      printf ("Error 3 on mpfr_cmp_ui(x,17) in check_macros\n"
+              "(c = %d instead of 1)\n", c);
+      exit (1);
+    }
+  if (mpfr_cmp_si (x, 17) != 0)
+    {
+      printf ("Error 1 on mpfr_cmp_si(x,17) in check_macros\n");
+      exit (1);
+    }
+  if (mpfr_cmp_si (x, (c++, 17)) != 0)
+    {
+      printf ("Error 2 on mpfr_cmp_si(x,17) in check_macros\n");
+      exit (1);
+    }
+  if (c != 2)
+    {
+      printf ("Error 3 on mpfr_cmp_si(x,17) in check_macros\n"
+              "(c = %d instead of 2)\n", c);
+      exit (1);
+    }
+
+  c = 0;
+  mpfr_set_ui (x, 0, MPFR_RNDN);
+  if (mpfr_cmp_ui (x, 0) != 0)
+    {
+      printf ("Error 1 on mpfr_cmp_ui(x,0) in check_macros\n");
+      exit (1);
+    }
+  if (mpfr_cmp_ui (x, (c++, 0)) != 0)
+    {
+      printf ("Error 2 on mpfr_cmp_ui(x,0) in check_macros\n");
+      exit (1);
+    }
+  if (c != 1)
+    {
+      printf ("Error 3 on mpfr_cmp_ui(x,0) in check_macros\n"
+              "(c = %d instead of 1)\n", c);
+      exit (1);
+    }
+  if (mpfr_cmp_si (x, 0) != 0)
+    {
+      printf ("Error 1 on mpfr_cmp_si(x,0) in check_macros\n");
+      exit (1);
+    }
+  if (mpfr_cmp_si (x, (c++, 0)) != 0)
+    {
+      printf ("Error 2 on mpfr_cmp_si(x,0) in check_macros\n");
+      exit (1);
+    }
+  if (c != 2)
+    {
+      printf ("Error 3 on mpfr_cmp_si(x,0) in check_macros\n"
+              "(c = %d instead of 2)\n", c);
+      exit (1);
+    }
+
+  mpfr_clear (x);
+}
+
+/* Bug in r7114 */
+static void
+test_macros (void)
+{
+  mpfr_t x[3];
+  mpfr_ptr p;
+
+  mpfr_inits (x[0], x[1], x[2], (mpfr_ptr) 0);
+  mpfr_set_ui (x[0], 0, MPFR_RNDN);
+  p = x[0];
+  if (mpfr_cmp_ui (p++, 0) != 0)
+    {
+      printf ("Error in mpfr_cmp_ui macro: result should be 0.\n");
+      exit (1);
+    }
+  if (p != x[1])
+    {
+      printf ("Error in mpfr_cmp_ui macro: p - x[0] = %d (expecting 1)\n",
+              (int) (p - x[0]));
+      exit (1);
+    }
+  p = x[0];
+  if (mpfr_cmp_si (p++, 0) != 0)
+    {
+      printf ("Error in mpfr_cmp_si macro: result should be 0.\n");
+      exit (1);
+    }
+  if (p != x[1])
+    {
+      printf ("Error in mpfr_cmp_si macro: p - x[0] = %d (expecting 1)\n",
+              (int) (p - x[0]));
+      exit (1);
+    }
+  mpfr_clears (x[0], x[1], x[2], (mpfr_ptr) 0);
+}
+
 int
 main (void)
 {
@@ -100,7 +220,7 @@ main (void)
   mpfr_init(x);
 
   /* tests for cmp_ui */
-  mpfr_set_ui (x, 3, GMP_RNDZ);
+  mpfr_set_ui (x, 3, MPFR_RNDZ);
   if ((mpfr_cmp_ui) (x, i = 3) != 0)
     {
       printf ("Error in mpfr_cmp_ui(3.0, 3)\n");
@@ -116,14 +236,14 @@ main (void)
       printf ("Error in mpfr_cmp_ui(3.0,4)\n");
       exit (1);
     }
-  mpfr_set_ui (x, 0, GMP_RNDZ);
-  mpfr_neg (x, x, GMP_RNDZ);
+  mpfr_set_ui (x, 0, MPFR_RNDZ);
+  mpfr_neg (x, x, MPFR_RNDZ);
   if (mpfr_cmp_ui (x, i = 0))
     {
       printf ("Error in mpfr_cmp_ui(0.0,0)\n");
       exit (1);
     }
-  mpfr_set_ui (x, 1, GMP_RNDZ);
+  mpfr_set_ui (x, 1, MPFR_RNDZ);
   if (mpfr_cmp_ui (x, i = 0) == 0)
     {
       printf ("Error in mpfr_cmp_ui(1.0,0)\n");
@@ -143,15 +263,15 @@ main (void)
       exit (1);
     }
 
-  mpfr_set_si (x, -1, GMP_RNDN);
+  mpfr_set_si (x, -1, MPFR_RNDN);
   MPFR_ASSERTN(mpfr_cmp_ui (x, 1) < 0);
   MPFR_ASSERTN(mpfr_cmp_ui (x, 0) < 0);
 
-  mpfr_set_ui (x, 1, GMP_RNDN);
+  mpfr_set_ui (x, 1, MPFR_RNDN);
   MPFR_ASSERTN(mpfr_cmp_ui (x, 0) > 0);
 
   /* tests for cmp_si */
-  (mpfr_set_si) (x, -3, GMP_RNDZ);
+  (mpfr_set_si) (x, -3, MPFR_RNDZ);
   if ((mpfr_cmp_si) (x, s = -3) != 0)
     {
       printf ("Error in mpfr_cmp_si(-3.0,-3)\n");
@@ -182,19 +302,19 @@ main (void)
     }
 
   /* case b=0 */
-  mpfr_set_ui (x, 0, GMP_RNDZ);
+  mpfr_set_ui (x, 0, MPFR_RNDZ);
   MPFR_ASSERTN(mpfr_cmp_si (x, 0) == 0);
   MPFR_ASSERTN(mpfr_cmp_si (x, 1) < 0);
   MPFR_ASSERTN(mpfr_cmp_si (x, -1) > 0);
 
   /* case i=0 */
-  mpfr_set_ui (x, 1, GMP_RNDZ);
+  mpfr_set_ui (x, 1, MPFR_RNDZ);
   MPFR_ASSERTN(mpfr_cmp_si (x, 0) > 0);
-  mpfr_set_ui (x, 0, GMP_RNDZ);
+  mpfr_set_ui (x, 0, MPFR_RNDZ);
   MPFR_ASSERTN(mpfr_cmp_si (x, 0) == 0);
-  mpfr_neg (x, x, GMP_RNDZ);
+  mpfr_neg (x, x, MPFR_RNDZ);
   MPFR_ASSERTN(mpfr_cmp_si (x, 0) == 0);
-  mpfr_set_si (x, -1, GMP_RNDZ);
+  mpfr_set_si (x, -1, MPFR_RNDZ);
   MPFR_ASSERTN(mpfr_cmp_si (x, 0) < 0);
 
   /* case large x */
@@ -208,14 +328,16 @@ main (void)
   MPFR_ASSERTN(mpfr_cmp_si (x, -1) < 0);
 
   /* corner case */
-  mpfr_set_ui (x, 1, GMP_RNDZ);
-  mpfr_mul_2exp (x, x, BITS_PER_MP_LIMB - 1, GMP_RNDZ);
-  /* now EXP(x)=BITS_PER_MP_LIMB */
+  mpfr_set_ui (x, 1, MPFR_RNDZ);
+  mpfr_mul_2exp (x, x, GMP_NUMB_BITS - 1, MPFR_RNDZ);
+  /* now EXP(x)=GMP_NUMB_BITS */
   MPFR_ASSERTN(mpfr_cmp_si (x, 1) > 0);
 
   mpfr_clear (x);
 
   check_nan ();
+  check_macros ();
+  test_macros ();
 
   tests_end_mpfr ();
   return 0;
