@@ -1,24 +1,24 @@
 /* Test file for mpfr_factorial.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,7 +38,7 @@ special (void)
 
   mpfr_set_prec (x, 21);
   mpfr_set_prec (y, 21);
-  mpfr_fac_ui (x, 119, GMP_RNDZ);
+  mpfr_fac_ui (x, 119, MPFR_RNDZ);
   mpfr_set_str_binary (y, "0.101111101110100110110E654");
   if (mpfr_cmp (x, y))
     {
@@ -47,7 +47,7 @@ special (void)
     }
 
   mpfr_set_prec (y, 206);
-  inex = mpfr_fac_ui (y, 767, GMP_RNDN);
+  inex = mpfr_fac_ui (y, 767, MPFR_RNDN);
   mpfr_set_prec (x, 206);
   mpfr_set_str_binary (x, "0.110111100001000001101010010001000111000100000100111000010011100011011111001100011110101000111101101100110001001100110100001001111110000101010000100100011100010011101110000001000010001100010000101001111E6250");
   if (mpfr_cmp (x, y))
@@ -62,7 +62,7 @@ special (void)
     }
 
   mpfr_set_prec (y, 202);
-  mpfr_fac_ui (y, 69, GMP_RNDU);
+  mpfr_fac_ui (y, 69, MPFR_RNDU);
 
   mpfr_clear (x);
   mpfr_clear (y);
@@ -74,7 +74,7 @@ test_int (void)
   unsigned long n0 = 1, n1 = 80, n;
   mpz_t f;
   mpfr_t x, y;
-  mp_prec_t prec_f, p;
+  mpfr_prec_t prec_f, p;
   int r;
   int inex1, inex2;
 
@@ -91,21 +91,21 @@ test_int (void)
         {
           mpfr_set_prec (x, p);
           mpfr_set_prec (y, p);
-          for (r = 0; r < GMP_RND_MAX; r++)
+          for (r = 0; r < MPFR_RND_MAX; r++)
             {
-              inex1 = mpfr_fac_ui (x, n, (mp_rnd_t) r);
-              inex2 = mpfr_set_z (y, f, (mp_rnd_t) r);
+              inex1 = mpfr_fac_ui (x, n, (mpfr_rnd_t) r);
+              inex2 = mpfr_set_z (y, f, (mpfr_rnd_t) r);
               if (mpfr_cmp (x, y))
                 {
                   printf ("Error for n=%lu prec=%lu rnd=%s\n",
-                          n, (unsigned long) p, mpfr_print_rnd_mode ((mp_rnd_t) r));
+                          n, (unsigned long) p, mpfr_print_rnd_mode ((mpfr_rnd_t) r));
                   exit (1);
                 }
               if ((inex1 < 0 && inex2 >= 0) || (inex1 == 0 && inex2 != 0)
                   || (inex1 > 0 && inex2 <= 0))
                 {
                   printf ("Wrong inexact flag for n=%lu prec=%lu rnd=%s\n",
-                          n, (unsigned long) p, mpfr_print_rnd_mode ((mp_rnd_t) r));
+                          n, (unsigned long) p, mpfr_print_rnd_mode ((mpfr_rnd_t) r));
                   exit (1);
                 }
             }
@@ -122,40 +122,40 @@ overflowed_fac0 (void)
 {
   mpfr_t x, y;
   int inex, rnd, err = 0;
-  mp_exp_t old_emax;
+  mpfr_exp_t old_emax;
 
   old_emax = mpfr_get_emax ();
 
   mpfr_init2 (x, 8);
   mpfr_init2 (y, 8);
 
-  mpfr_set_ui (y, 1, GMP_RNDN);
+  mpfr_set_ui (y, 1, MPFR_RNDN);
   mpfr_nextbelow (y);
   set_emax (0);  /* 1 is not representable. */
   RND_LOOP (rnd)
     {
       mpfr_clear_flags ();
-      inex = mpfr_fac_ui (x, 0, rnd);
+      inex = mpfr_fac_ui (x, 0, (mpfr_rnd_t) rnd);
       if (! mpfr_overflow_p ())
         {
           printf ("Error in overflowed_fac0 (rnd = %s):\n"
                   "  The overflow flag is not set.\n",
-                  mpfr_print_rnd_mode (rnd));
+                  mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
           err = 1;
         }
-      if (rnd == GMP_RNDZ || rnd == GMP_RNDD)
+      if (rnd == MPFR_RNDZ || rnd == MPFR_RNDD)
         {
           if (inex >= 0)
             {
               printf ("Error in overflowed_fac0 (rnd = %s):\n"
                       "  The inexact value must be negative.\n",
-                      mpfr_print_rnd_mode (rnd));
+                      mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
               err = 1;
             }
           if (! mpfr_equal_p (x, y))
             {
               printf ("Error in overflowed_fac0 (rnd = %s):\n"
-                      "  Got ", mpfr_print_rnd_mode (rnd));
+                      "  Got ", mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
               mpfr_print_binary (x);
               printf (" instead of 0.11111111E0.\n");
               err = 1;
@@ -167,13 +167,13 @@ overflowed_fac0 (void)
             {
               printf ("Error in overflowed_fac0 (rnd = %s):\n"
                       "  The inexact value must be positive.\n",
-                      mpfr_print_rnd_mode (rnd));
+                      mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
               err = 1;
             }
           if (! (mpfr_inf_p (x) && MPFR_SIGN (x) > 0))
             {
               printf ("Error in overflowed_fac0 (rnd = %s):\n"
-                      "  Got ", mpfr_print_rnd_mode (rnd));
+                      "  Got ", mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
               mpfr_print_binary (x);
               printf (" instead of +Inf.\n");
               err = 1;
@@ -207,7 +207,7 @@ main (int argc, char *argv[])
   mpfr_init (z);
   mpfr_init (t);
 
-  mpfr_fac_ui (y, 0, GMP_RNDN);
+  mpfr_fac_ui (y, 0, MPFR_RNDN);
 
   if (mpfr_cmp_ui (y, 1))
     {
@@ -224,18 +224,18 @@ main (int argc, char *argv[])
       mpfr_set_prec (y, yprec);
 
       for (n = 0; n < 50; n++)
-        for (rnd = 0; rnd < GMP_RND_MAX; rnd++)
+        for (rnd = 0; rnd < MPFR_RND_MAX; rnd++)
           {
-            inexact = mpfr_fac_ui (y, n, (mp_rnd_t) rnd);
-            err = (rnd == GMP_RNDN) ? yprec + 1 : yprec;
-            if (mpfr_can_round (y, err, (mp_rnd_t) rnd, (mp_rnd_t) rnd, prec))
+            inexact = mpfr_fac_ui (y, n, (mpfr_rnd_t) rnd);
+            err = (rnd == MPFR_RNDN) ? yprec + 1 : yprec;
+            if (mpfr_can_round (y, err, (mpfr_rnd_t) rnd, (mpfr_rnd_t) rnd, prec))
               {
-                mpfr_set (t, y, (mp_rnd_t) rnd);
-                inexact = mpfr_fac_ui (z, n, (mp_rnd_t) rnd);
+                mpfr_set (t, y, (mpfr_rnd_t) rnd);
+                inexact = mpfr_fac_ui (z, n, (mpfr_rnd_t) rnd);
                 /* fact(n) ends with floor(n/2)+floor(n/4)+... zeros */
                 for (k=n/2, zeros=0; k; k >>= 1)
                   zeros += k;
-                if (MPFR_EXP(y) <= (mp_exp_t) (prec + zeros))
+                if (MPFR_EXP(y) <= (mpfr_exp_t) (prec + zeros))
                   /* result should be exact */
                   {
                     if (inexact)
@@ -257,14 +257,14 @@ main (int argc, char *argv[])
                 if (mpfr_cmp (t, z))
                   {
                     printf ("results differ for x=");
-                    mpfr_out_str (stdout, 2, prec, x, GMP_RNDN);
+                    mpfr_out_str (stdout, 2, prec, x, MPFR_RNDN);
                     printf (" prec=%u rnd_mode=%s\n", prec,
-                            mpfr_print_rnd_mode ((mp_rnd_t) rnd));
+                            mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
                     printf ("   got ");
-                    mpfr_out_str (stdout, 2, prec, z, GMP_RNDN);
+                    mpfr_out_str (stdout, 2, prec, z, MPFR_RNDN);
                     puts ("");
                     printf ("   expected ");
-                    mpfr_out_str (stdout, 2, prec, t, GMP_RNDN);
+                    mpfr_out_str (stdout, 2, prec, t, MPFR_RNDN);
                     puts ("");
                     printf ("   approximation was ");
                     mpfr_print_binary (y);
