@@ -1,24 +1,24 @@
 /* Test file for mpfr_fma.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,7 +29,7 @@ MA 02110-1301, USA. */
 static void
 test_exact (void)
 {
-  char *val[] =
+  const char *val[] =
     { "@NaN@", "-@Inf@", "-2", "-1", "-0", "0", "1", "2", "@Inf@" };
   int sv = sizeof (val) / sizeof (*val);
   int i, j, k;
@@ -43,17 +43,17 @@ test_exact (void)
       for (k = 0; k < sv; k++)
         RND_LOOP (rnd)
           {
-            if (mpfr_set_str (a, val[i], 10, GMP_RNDN) ||
-                mpfr_set_str (b, val[j], 10, GMP_RNDN) ||
-                mpfr_set_str (c, val[k], 10, GMP_RNDN) ||
-                mpfr_mul (r1, a, b, rnd) ||
-                mpfr_add (r1, r1, c, rnd))
+            if (mpfr_set_str (a, val[i], 10, MPFR_RNDN) ||
+                mpfr_set_str (b, val[j], 10, MPFR_RNDN) ||
+                mpfr_set_str (c, val[k], 10, MPFR_RNDN) ||
+                mpfr_mul (r1, a, b, (mpfr_rnd_t) rnd) ||
+                mpfr_add (r1, r1, c, (mpfr_rnd_t) rnd))
               {
                 printf ("test_exact internal error for (%d,%d,%d,%d)\n",
                         i, j, k, rnd);
                 exit (1);
               }
-            if (mpfr_fma (r2, a, b, c, rnd))
+            if (mpfr_fma (r2, a, b, c, (mpfr_rnd_t) rnd))
               {
                 printf ("test_exact(%d,%d,%d,%d): mpfr_fma should be exact\n",
                         i, j, k, rnd);
@@ -70,9 +70,9 @@ test_exact (void)
             if (mpfr_cmp (r1, r2) || MPFR_SIGN (r1) != MPFR_SIGN (r2))
               {
                 printf ("test_exact(%d,%d,%d,%d):\nexpected ", i, j, k, rnd);
-                mpfr_out_str (stdout, 10, 0, r1, GMP_RNDN);
+                mpfr_out_str (stdout, 10, 0, r1, MPFR_RNDN);
                 printf ("\n     got ");
-                mpfr_out_str (stdout, 10, 0, r2, GMP_RNDN);
+                mpfr_out_str (stdout, 10, 0, r2, MPFR_RNDN);
                 printf ("\n");
                 exit (1);
               }
@@ -90,18 +90,18 @@ test_overflow1 (void)
   mpfr_inits2 (8, x, y, z, r, (mpfr_ptr) 0);
   MPFR_SET_POS (x);
   mpfr_setmax (x, mpfr_get_emax ());  /* x = 2^emax - ulp */
-  mpfr_set_ui (y, 2, GMP_RNDN);       /* y = 2 */
-  mpfr_neg (z, x, GMP_RNDN);          /* z = -x = -(2^emax - ulp) */
+  mpfr_set_ui (y, 2, MPFR_RNDN);       /* y = 2 */
+  mpfr_neg (z, x, MPFR_RNDN);          /* z = -x = -(2^emax - ulp) */
   mpfr_clear_flags ();
   /* The intermediate multiplication x * y overflows, but x * y + z = x
      is representable. */
-  inex = mpfr_fma (r, x, y, z, GMP_RNDN);
+  inex = mpfr_fma (r, x, y, z, MPFR_RNDN);
   if (inex || ! mpfr_equal_p (r, x))
     {
       printf ("Error in test_overflow1\nexpected ");
-      mpfr_out_str (stdout, 2, 0, x, GMP_RNDN);
+      mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
       printf (" with inex = 0\n     got ");
-      mpfr_out_str (stdout, 2, 0, r, GMP_RNDN);
+      mpfr_out_str (stdout, 2, 0, r, MPFR_RNDN);
       printf (" with inex = %d\n", inex);
       exit (1);
     }
@@ -123,7 +123,7 @@ test_overflow2 (void)
 
   MPFR_SET_POS (x);
   mpfr_setmin (x, mpfr_get_emax ());  /* x = 0.1@emax */
-  mpfr_set_si (y, -2, GMP_RNDN);      /* y = -2 */
+  mpfr_set_si (y, -2, MPFR_RNDN);      /* y = -2 */
   /* The intermediate multiplication x * y will overflow. */
 
   for (i = -9; i <= 9; i++)
@@ -131,44 +131,45 @@ test_overflow2 (void)
       {
         int inf, overflow;
 
-        inf = rnd == GMP_RNDN || rnd == GMP_RNDD;
+        inf = rnd == MPFR_RNDN || rnd == MPFR_RNDD || rnd == MPFR_RNDA;
         overflow = inf || i <= 0;
 
-        inex = mpfr_set_si_2exp (z, i, mpfr_get_emin (), GMP_RNDN);
+        inex = mpfr_set_si_2exp (z, i, mpfr_get_emin (), MPFR_RNDN);
         MPFR_ASSERTN (inex == 0);
 
         mpfr_clear_flags ();
         /* One has: x * y = -1@emax exactly (but not representable). */
-        inex = mpfr_fma (r, x, y, z, rnd);
+        inex = mpfr_fma (r, x, y, z, (mpfr_rnd_t) rnd);
         if (overflow ^ (mpfr_overflow_p () != 0))
           {
             printf ("Error in test_overflow2 (i = %d, %s): wrong overflow"
-                    " flag (should be %d)\n", i, mpfr_print_rnd_mode (rnd),
-                    overflow);
+                    " flag (should be %d)\n", i,
+                    mpfr_print_rnd_mode ((mpfr_rnd_t) rnd), overflow);
             err = 1;
           }
         if (mpfr_nanflag_p ())
           {
             printf ("Error in test_overflow2 (i = %d, %s): NaN flag should"
-                    " not be set\n", i, mpfr_print_rnd_mode (rnd));
+                    " not be set\n", i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
             err = 1;
           }
         if (mpfr_nan_p (r))
           {
             printf ("Error in test_overflow2 (i = %d, %s): got NaN\n",
-                    i, mpfr_print_rnd_mode (rnd));
+                    i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
             err = 1;
           }
         else if (MPFR_SIGN (r) >= 0)
           {
             printf ("Error in test_overflow2 (i = %d, %s): wrong sign "
-                    "(+ instead of -)\n", i, mpfr_print_rnd_mode (rnd));
+                    "(+ instead of -)\n", i,
+                    mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
             err = 1;
           }
         else if (inf && ! mpfr_inf_p (r))
           {
             printf ("Error in test_overflow2 (i = %d, %s): expected -Inf,"
-                    " got\n", i, mpfr_print_rnd_mode (rnd));
+                    " got\n", i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
             mpfr_dump (r);
             err = 1;
           }
@@ -176,14 +177,15 @@ test_overflow2 (void)
                           (mpfr_nextbelow (r), ! mpfr_inf_p (r))))
           {
             printf ("Error in test_overflow2 (i = %d, %s): expected -MAX,"
-                    " got\n", i, mpfr_print_rnd_mode (rnd));
+                    " got\n", i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
             mpfr_dump (r);
             err = 1;
           }
         if (inf ? inex >= 0 : inex <= 0)
           {
             printf ("Error in test_overflow2 (i = %d, %s): wrong inexact"
-                    " flag (got %d)\n", i, mpfr_print_rnd_mode (rnd), inex);
+                    " flag (got %d)\n", i,
+                    mpfr_print_rnd_mode ((mpfr_rnd_t) rnd), inex);
             err = 1;
           }
 
@@ -207,66 +209,66 @@ test_underflow1 (void)
 
   for (signy = -1; signy <= 1; signy += 2)
     {
-      mpfr_set_si_2exp (y, signy, -1, GMP_RNDN);  /* |y| = 1/2 */
+      mpfr_set_si_2exp (y, signy, -1, MPFR_RNDN);  /* |y| = 1/2 */
       for (signz = -3; signz <= 3; signz += 2)
         {
           RND_LOOP (rnd)
             {
-              mpfr_set_si (z, signz, GMP_RNDN);
+              mpfr_set_si (z, signz, MPFR_RNDN);
               if (ABS (signz) != 1)
                 mpfr_setmax (z, mpfr_get_emax ());
               /* |z| = 1 or 2^emax - ulp */
               mpfr_clear_flags ();
-              inex = mpfr_fma (r, x, y, z, rnd);
+              inex = mpfr_fma (r, x, y, z, (mpfr_rnd_t) rnd);
 #define ERRTU1 "Error in test_underflow1 (signy = %d, signz = %d, %s)\n  "
               if (mpfr_nanflag_p ())
                 {
                   printf (ERRTU1 "NaN flag is set\n", signy, signz,
-                          mpfr_print_rnd_mode (rnd));
+                          mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
                   err = 1;
                 }
-              if (signy < 0 && (rnd == GMP_RNDD ||
-                                (rnd == GMP_RNDZ && signz > 0)))
+              if (signy < 0 && MPFR_IS_LIKE_RNDD(rnd, signz))
                 mpfr_nextbelow (z);
-              if (signy > 0 && (rnd == GMP_RNDU ||
-                                (rnd == GMP_RNDZ && signz < 0)))
+              if (signy > 0 && MPFR_IS_LIKE_RNDU(rnd, signz))
                 mpfr_nextabove (z);
               if ((mpfr_overflow_p () != 0) ^ (mpfr_inf_p (z) != 0))
                 {
                   printf (ERRTU1 "wrong overflow flag\n", signy, signz,
-                          mpfr_print_rnd_mode (rnd));
+                          mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
                   err = 1;
                 }
               if (mpfr_underflow_p ())
                 {
                   printf (ERRTU1 "underflow flag is set\n", signy, signz,
-                          mpfr_print_rnd_mode (rnd));
+                          mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
                   err = 1;
                 }
               if (! mpfr_equal_p (r, z))
                 {
                   printf (ERRTU1 "got ", signy, signz,
-                          mpfr_print_rnd_mode (rnd));
+                          mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
                   mpfr_print_binary (r);
                   printf (" instead of ");
                   mpfr_print_binary (z);
                   printf ("\n");
                   err = 1;
                 }
-              if (inex >= 0 && (rnd == GMP_RNDD ||
-                                (rnd == GMP_RNDZ && signz > 0) ||
-                                (rnd == GMP_RNDN && signy > 0)))
+              if (inex >= 0 && (rnd == MPFR_RNDD ||
+                                (rnd == MPFR_RNDZ && signz > 0) ||
+                                (rnd == MPFR_RNDN && signy > 0)))
                 {
                   printf (ERRTU1 "ternary value = %d instead of < 0\n",
-                          signy, signz, mpfr_print_rnd_mode (rnd), inex);
+                          signy, signz, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd),
+                          inex);
                   err = 1;
                 }
-              if (inex <= 0 && (rnd == GMP_RNDU ||
-                                (rnd == GMP_RNDZ && signz < 0) ||
-                                (rnd == GMP_RNDN && signy < 0)))
+              if (inex <= 0 && (rnd == MPFR_RNDU ||
+                                (rnd == MPFR_RNDZ && signz < 0) ||
+                                (rnd == MPFR_RNDN && signy < 0)))
                 {
                   printf (ERRTU1 "ternary value = %d instead of > 0\n",
-                          signy, signz, mpfr_print_rnd_mode (rnd), inex);
+                          signy, signz, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd),
+                          inex);
                   err = 1;
                 }
             }
@@ -286,21 +288,21 @@ test_underflow2 (void)
 
   mpfr_inits2 (32, x, y, z, r, (mpfr_ptr) 0);
 
-  mpfr_set_si_2exp (z, 1, mpfr_get_emin (), GMP_RNDN);   /* z = 2^emin */
-  mpfr_set_si_2exp (x, 1, mpfr_get_emin (), GMP_RNDN);   /* x = 2^emin */
+  mpfr_set_si_2exp (z, 1, mpfr_get_emin (), MPFR_RNDN);   /* z = 2^emin */
+  mpfr_set_si_2exp (x, 1, mpfr_get_emin (), MPFR_RNDN);   /* x = 2^emin */
 
   for (b = 0; b <= 1; b++)
     {
       for (i = 15; i <= 17; i++)
         {
-          mpfr_set_si_2exp (y, i, -4 - MPFR_PREC (z), GMP_RNDN);
+          mpfr_set_si_2exp (y, i, -4 - MPFR_PREC (z), MPFR_RNDN);
           /*  z = 1.000...00b
            * xy =            01111
            *   or            10000
            *   or            10001
            */
           mpfr_clear_flags ();
-          inex = mpfr_fma (r, x, y, z, GMP_RNDN);
+          inex = mpfr_fma (r, x, y, z, MPFR_RNDN);
 #define ERRTU2 "Error in test_underflow2 (b = %d, i = %d)\n  "
           if (__gmpfr_flags != MPFR_FLAGS_INEXACT)
             {
@@ -315,7 +317,7 @@ test_underflow2 (void)
                       b, i, inex, same ? '<' : '>');
               err = 1;
             }
-          mpfr_set (y, z, GMP_RNDN);
+          mpfr_set (y, z, MPFR_RNDN);
           if (!same)
             mpfr_nextabove (y);
           if (! mpfr_equal_p (r, y))
@@ -335,13 +337,104 @@ test_underflow2 (void)
   mpfr_clears (x, y, z, r, (mpfr_ptr) 0);
 }
 
+static void
+bug20101018 (void)
+{
+  mpfr_t x, y, z, t, u;
+  int i;
+
+  mpfr_init2 (x, 64);
+  mpfr_init2 (y, 64);
+  mpfr_init2 (z, 64);
+  mpfr_init2 (t, 64);
+  mpfr_init2 (u, 64);
+
+  mpfr_set_str (x, "0xf.fffffffffffffffp-14766", 16, MPFR_RNDN);
+  mpfr_set_str (y, "-0xf.fffffffffffffffp+317", 16, MPFR_RNDN);
+  mpfr_set_str (z, "0x8.3ffffffffffe3ffp-14443", 16, MPFR_RNDN);
+  mpfr_set_str (t, "0x8.7ffffffffffc7ffp-14444", 16, MPFR_RNDN);
+  i = mpfr_fma (u, x, y, z, MPFR_RNDN);
+  if (mpfr_cmp (u, t) != 0)
+    {
+      printf ("Wrong result in bug20101018 (a)\n");
+      printf ("Expected ");
+      mpfr_out_str (stdout, 16, 0, t, MPFR_RNDN);
+      printf ("\nGot      ");
+      mpfr_out_str (stdout, 16, 0, u, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+  if (i <= 0)
+    {
+      printf ("Wrong ternary value in bug20101018 (a)\n");
+      printf ("Expected > 0\n");
+      printf ("Got      %d\n", i);
+      exit (1);
+    }
+
+  mpfr_set_str (x, "-0xf.fffffffffffffffp-11420", 16, MPFR_RNDN);
+  mpfr_set_str (y, "0xf.fffffffffffffffp+9863", 16, MPFR_RNDN);
+  mpfr_set_str (z, "0x8.fffff80ffffffffp-1551", 16, MPFR_RNDN);
+  mpfr_set_str (t, "0x9.fffff01ffffffffp-1552", 16, MPFR_RNDN);
+  i = mpfr_fma (u, x, y, z, MPFR_RNDN);
+  if (mpfr_cmp (u, t) != 0)
+    {
+      printf ("Wrong result in bug20101018 (b)\n");
+      printf ("Expected ");
+      mpfr_out_str (stdout, 16, 0, t, MPFR_RNDN);
+      printf ("\nGot      ");
+      mpfr_out_str (stdout, 16, 0, u, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+  if (i <= 0)
+    {
+      printf ("Wrong ternary value in bug20101018 (b)\n");
+      printf ("Expected > 0\n");
+      printf ("Got      %d\n", i);
+      exit (1);
+    }
+
+  mpfr_set_str (x, "0xf.fffffffffffffffp-2125", 16, MPFR_RNDN);
+  mpfr_set_str (y, "-0xf.fffffffffffffffp-6000", 16, MPFR_RNDN);
+  mpfr_set_str (z, "0x8p-8119", 16, MPFR_RNDN);
+  mpfr_set_str (t, "0x8.000000000000001p-8120", 16, MPFR_RNDN);
+  i = mpfr_fma (u, x, y, z, MPFR_RNDN);
+  if (mpfr_cmp (u, t) != 0)
+    {
+      printf ("Wrong result in bug20101018 (c)\n");
+      printf ("Expected ");
+      mpfr_out_str (stdout, 16, 0, t, MPFR_RNDN);
+      printf ("\nGot      ");
+      mpfr_out_str (stdout, 16, 0, u, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+  if (i <= 0)
+    {
+      printf ("Wrong ternary value in bug20101018 (c)\n");
+      printf ("Expected > 0\n");
+      printf ("Got      %d\n", i);
+      exit (1);
+    }
+
+  mpfr_clear (x);
+  mpfr_clear (y);
+  mpfr_clear (z);
+  mpfr_clear (t);
+  mpfr_clear (u);
+}
+
 int
 main (int argc, char *argv[])
 {
   mpfr_t x, y, z, s;
+  MPFR_SAVE_EXPO_DECL (expo);
 
   tests_start_mpfr ();
 
+  bug20101018 ();
+
   mpfr_init (x);
   mpfr_init (s);
   mpfr_init (y);
@@ -352,10 +445,10 @@ main (int argc, char *argv[])
   mpfr_set_prec (y, 2);
   mpfr_set_prec (z, 2);
   mpfr_set_prec (s, 2);
-  mpfr_set_str (x, "-0.75", 10, GMP_RNDN);
-  mpfr_set_str (y, "0.5", 10, GMP_RNDN);
-  mpfr_set_str (z, "0.375", 10, GMP_RNDN);
-  mpfr_fma (s, x, y, z, GMP_RNDU); /* result is 0 */
+  mpfr_set_str (x, "-0.75", 10, MPFR_RNDN);
+  mpfr_set_str (y, "0.5", 10, MPFR_RNDN);
+  mpfr_set_str (z, "0.375", 10, MPFR_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDU); /* result is 0 */
   if (mpfr_cmp_ui(s, 0))
     {
       printf("Error: -0.75 * 0.5 + 0.375 should be equal to 0 for prec=2\n");
@@ -367,18 +460,18 @@ main (int argc, char *argv[])
   mpfr_set_prec (z, 27);
   mpfr_set_prec (s, 27);
   mpfr_set_str_binary (x, "1.11111111111111111111111111e-1");
-  mpfr_set (y, x, GMP_RNDN);
+  mpfr_set (y, x, MPFR_RNDN);
   mpfr_set_str_binary (z, "-1.00011110100011001011001001e-1");
-  if (mpfr_fma (s, x, y, z, GMP_RNDN) >= 0)
+  if (mpfr_fma (s, x, y, z, MPFR_RNDN) >= 0)
     {
       printf ("Wrong inexact flag for x=y=1-2^(-27)\n");
       exit (1);
     }
 
   mpfr_set_nan (x);
-  mpfr_random (y);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_urandomb (y, RANDS);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p (s))
     {
       printf ("evaluation of function in x=NAN does not return NAN");
@@ -386,9 +479,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_nan (y);
-  mpfr_random (x);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_urandomb (x, RANDS);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p(s))
     {
       printf ("evaluation of function in y=NAN does not return NAN");
@@ -396,9 +489,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_nan (z);
-  mpfr_random (y);
-  mpfr_random (x);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_urandomb (y, RANDS);
+  mpfr_urandomb (x, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p (s))
     {
       printf ("evaluation of function in z=NAN does not return NAN");
@@ -408,7 +501,7 @@ main (int argc, char *argv[])
   mpfr_set_inf (x, 1);
   mpfr_set_inf (y, 1);
   mpfr_set_inf (z, 1);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) < 0)
     {
       printf ("Error for (+inf) * (+inf) + (+inf)\n");
@@ -418,7 +511,7 @@ main (int argc, char *argv[])
   mpfr_set_inf (x, -1);
   mpfr_set_inf (y, -1);
   mpfr_set_inf (z, 1);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) < 0)
     {
       printf ("Error for (-inf) * (-inf) + (+inf)\n");
@@ -428,7 +521,7 @@ main (int argc, char *argv[])
   mpfr_set_inf (x, 1);
   mpfr_set_inf (y, -1);
   mpfr_set_inf (z, -1);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) > 0)
     {
       printf ("Error for (+inf) * (-inf) + (-inf)\n");
@@ -438,7 +531,7 @@ main (int argc, char *argv[])
   mpfr_set_inf (x, -1);
   mpfr_set_inf (y, 1);
   mpfr_set_inf (z, -1);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) > 0)
     {
       printf ("Error for (-inf) * (+inf) + (-inf)\n");
@@ -446,9 +539,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (x, 1);
-  mpfr_set_ui (y, 0, GMP_RNDN);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_set_ui (y, 0, MPFR_RNDN);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p (s))
     {
       printf ("evaluation of function in x=INF y=0  does not return NAN");
@@ -456,9 +549,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (y, 1);
-  mpfr_set_ui (x, 0, GMP_RNDN);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_set_ui (x, 0, MPFR_RNDN);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p (s))
     {
       printf ("evaluation of function in x=0 y=INF does not return NAN");
@@ -466,9 +559,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (x, 1);
-  mpfr_random (y); /* always positive */
+  mpfr_urandomb (y, RANDS); /* always positive */
   mpfr_set_inf (z, -1);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p (s))
     {
       printf ("evaluation of function in x=INF y>0 z=-INF does not return NAN");
@@ -476,9 +569,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (y, 1);
-  mpfr_random (x);
+  mpfr_urandomb (x, RANDS);
   mpfr_set_inf (z, -1);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_nan_p (s))
     {
       printf ("evaluation of function in x>0 y=INF z=-INF does not return NAN");
@@ -486,9 +579,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (x, 1);
-  mpfr_random (y);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_urandomb (y, RANDS);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) < 0)
     {
       printf ("evaluation of function in x=INF does not return INF");
@@ -496,9 +589,9 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (y, 1);
-  mpfr_random (x);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_urandomb (x, RANDS);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) < 0)
     {
       printf ("evaluation of function in y=INF does not return INF");
@@ -506,29 +599,29 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_inf (z, 1);
-  mpfr_random (x);
-  mpfr_random (y);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_urandomb (x, RANDS);
+  mpfr_urandomb (y, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (!mpfr_inf_p (s) || mpfr_sgn (s) < 0)
     {
       printf ("evaluation of function in z=INF does not return INF");
       exit (1);
     }
 
-  mpfr_set_ui (x, 0, GMP_RNDN);
-  mpfr_random (y);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_set_ui (x, 0, MPFR_RNDN);
+  mpfr_urandomb (y, RANDS);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (mpfr_cmp (s, z))
     {
       printf ("evaluation of function in x=0 does not return z\n");
       exit (1);
     }
 
-  mpfr_set_ui (y, 0, GMP_RNDN);
-  mpfr_random (x);
-  mpfr_random (z);
-  mpfr_fma (s, x, y, z, GMP_RNDN);
+  mpfr_set_ui (y, 0, MPFR_RNDN);
+  mpfr_urandomb (x, RANDS);
+  mpfr_urandomb (z, RANDS);
+  mpfr_fma (s, x, y, z, MPFR_RNDN);
   if (mpfr_cmp (s, z))
     {
       printf ("evaluation of function in y=0 does not return z\n");
@@ -536,13 +629,13 @@ main (int argc, char *argv[])
     }
 
   {
-    mp_prec_t prec;
+    mpfr_prec_t prec;
     mpfr_t t, slong;
-    mp_rnd_t rnd;
+    mpfr_rnd_t rnd;
     int inexact, compare;
     unsigned int n;
 
-    mp_prec_t p0=2, p1=200;
+    mpfr_prec_t p0=2, p1=200;
     unsigned int N=200;
 
     mpfr_init (t);
@@ -559,18 +652,18 @@ main (int argc, char *argv[])
 
       for (n=0; n<N; n++)
         {
-          mpfr_random (x);
-          mpfr_random (y);
-          mpfr_random (z);
+          mpfr_urandomb (x, RANDS);
+          mpfr_urandomb (y, RANDS);
+          mpfr_urandomb (z, RANDS);
 
           if (randlimb () % 2)
-            mpfr_neg (x, x, GMP_RNDN);
+            mpfr_neg (x, x, MPFR_RNDN);
           if (randlimb () % 2)
-            mpfr_neg (y, y, GMP_RNDN);
+            mpfr_neg (y, y, MPFR_RNDN);
           if (randlimb () % 2)
-            mpfr_neg (z, z, GMP_RNDN);
+            mpfr_neg (z, z, MPFR_RNDN);
 
-          rnd = (mp_rnd_t) RND_RAND ();
+          rnd = RND_RAND ();
           mpfr_set_prec (slong, 2 * prec);
           if (mpfr_mul (slong, x, y, rnd))
             {
@@ -582,18 +675,18 @@ main (int argc, char *argv[])
           if (mpfr_cmp (s, t))
             {
               printf ("results differ for x=");
-              mpfr_out_str (stdout, 2, prec, x, GMP_RNDN);
+              mpfr_out_str (stdout, 2, prec, x, MPFR_RNDN);
               printf ("  y=");
-              mpfr_out_str (stdout, 2, prec, y, GMP_RNDN);
+              mpfr_out_str (stdout, 2, prec, y, MPFR_RNDN);
               printf ("  z=");
-              mpfr_out_str (stdout, 2, prec, z, GMP_RNDN);
+              mpfr_out_str (stdout, 2, prec, z, MPFR_RNDN);
               printf (" prec=%u rnd_mode=%s\n", (unsigned int) prec,
                       mpfr_print_rnd_mode (rnd));
               printf ("got      ");
-              mpfr_out_str (stdout, 2, prec, s, GMP_RNDN);
+              mpfr_out_str (stdout, 2, prec, s, MPFR_RNDN);
               puts ("");
               printf ("expected ");
-              mpfr_out_str (stdout, 2, prec, t, GMP_RNDN);
+              mpfr_out_str (stdout, 2, prec, t, MPFR_RNDN);
               puts ("");
               printf ("approx  ");
               mpfr_print_binary (slong);
@@ -606,10 +699,10 @@ main (int argc, char *argv[])
             {
               printf ("Wrong inexact flag for rnd=%s: expected %d, got %d\n",
                       mpfr_print_rnd_mode (rnd), compare, inexact);
-              printf (" x="); mpfr_out_str (stdout, 2, 0, x, GMP_RNDN);
-              printf (" y="); mpfr_out_str (stdout, 2, 0, y, GMP_RNDN);
-              printf (" z="); mpfr_out_str (stdout, 2, 0, z, GMP_RNDN);
-              printf (" s="); mpfr_out_str (stdout, 2, 0, s, GMP_RNDN);
+              printf (" x="); mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
+              printf (" y="); mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+              printf (" z="); mpfr_out_str (stdout, 2, 0, z, MPFR_RNDN);
+              printf (" s="); mpfr_out_str (stdout, 2, 0, s, MPFR_RNDN);
               printf ("\n");
               exit (1);
             }
@@ -625,10 +718,13 @@ main (int argc, char *argv[])
   mpfr_clear (s);
 
   test_exact ();
+
+  MPFR_SAVE_EXPO_MARK (expo);
   test_overflow1 ();
   test_overflow2 ();
   test_underflow1 ();
   test_underflow2 ();
+  MPFR_SAVE_EXPO_FREE (expo);
 
   tests_end_mpfr ();
   return 0;
